/*
 *
 * Copyright (C) 2001 Adrian Schroeter <adrian@suse.de>
 *
 */

#include <qwidget.h>
#include <qlayout.h>
#include <qcheckbox.h>
#include <qstring.h>
#include <qspinbox.h>
#include <qregexp.h>
#include <qlabel.h>
#include <qwhatsthis.h>
#include <qlayout.h>
#include <qtimer.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qdom.h>

#include <kapp.h>
#include <kconfig.h>
#include <kglobal.h>
#include <kstddirs.h>
#include <klocale.h>
#include <kcmodule.h>
#include <kprocess.h>
#include <kapplication.h>

#include "greetings.h"

#define GREET_HTML "greet.html"

#define DEFAULT_SUSETOUR "/usr/share/doc/susetour/en/index.html"
#define DEFAULT_RELEASENOTES "/usr/share/doc/release-notes/RELEASE-NOTES.en.html"


/**** SUSEgreetings ****/

SUSEgreetings::SUSEgreetings()
    : QDialog(0, "greetings", false, Qt::WStyle_NoBorder | Qt::WStyle_Customize )
{
    QString version;
    // which SUSE version do we run ?
    QFile f("/etc/SUSE-release");
    if ( f.open(IO_ReadOnly) ){
	f.readLine(version, 256);
	version = version.stripWhiteSpace();
	version.remove(QRegExp("\\([^)]*\\)$"));
	version = version.stripWhiteSpace();
	f.close();
    }else
        version="SUSE Linux";

    QString content;

    // header 
    QString filename = locate( "data", "SUSEgreeter/greet_header.html" );
    f.setName(filename);
    if ( ! f.open(IO_ReadOnly))
    {
	// %1 = filename
	content = i18n("Error loading %1").arg("greet_header.html");
    }
    else
    {
	QTextStream stream(&f);
	stream.setEncoding(QTextStream::UnicodeUTF8);
	content = stream.read();
	stream.reset();
	stream.unsetDevice();
	f.close();
    }

    // section template
    QString section_template;
    filename = locate( "data", "SUSEgreeter/greet_section.html" );
    f.setName(filename);
    if ( ! f.open(IO_ReadOnly))
    {
	// %1 = filename
	 section_template = i18n("Error loading %1").arg("greet_section.html");
    }
    else
    {
	QTextStream stream(&f);
	stream.setEncoding(QTextStream::UnicodeUTF8);
	section_template = stream.read();
	stream.reset();
	stream.unsetDevice();
	f.close();
    }

    // sections
    filename = locate( "data", "SUSEgreeter/greet.xml" );
    f.setName(filename);
    if ( ! f.open(IO_ReadOnly))
    {
	// %1 = filename
	content += i18n("Error loading %1").arg("greet.xml");
    }
    else
    {
        QDomDocument doc( "sections" );
        if ( !doc.setContent( &f )) {
           content += i18n("Error loading %1").arg("greet.xml");
        }
        else {
           QDomElement docElem = doc.documentElement();
           QDomNode n = docElem.firstChild();
           while( !n.isNull() ) {
               QDomElement e = n.toElement();
               if( !e.isNull() ) {
                   if (e.tagName()=="section") {
                      QString section = section_template;
                      QDomNode ex = n.firstChild();
                      while ( !ex.isNull() ) {
                         QDomElement e2 = ex.toElement();
                         if( !e2.isNull() ) {
                            if (e2.tagName()=="icon") {
				section.replace("__ICON__", e2.text() );
                            }
                            else if (e2.tagName()=="title") {
				section.replace("__TITLE__", i18n(e2.text()) );
}
                            else if (e2.tagName()=="text") {
				QString dummy = e2.text().replace('<',"&lt;").replace('>',"&gt;");
				section.replace("__TEXT__", i18n(dummy).replace("&lt;","<").replace("&gt;",">"));
                            }
                         }
                         ex = ex.nextSibling();
                      }
                      content += section;
                   }
               }
               n = n.nextSibling();
           }
        }
	f.close();
    }

    // footer
    filename = locate( "data", "SUSEgreeter/greet_footer.html" );
    f.setName(filename);
    if ( ! f.open(IO_ReadOnly))
    {
	// %1 = filename
	content = i18n("Error loading %1").arg("greet_footer.html");
    }
    else
    {
	QTextStream stream(&f);
	stream.setEncoding(QTextStream::UnicodeUTF8);
	content += stream.read();
	stream.reset();
	stream.unsetDevice();
	f.close();
    }
    content.replace( "__ENJOY_OPENSUSE__", i18n("Have a lot of fun!"));
    content.replace( "__CLOSE__", i18n("CLOSE"));


    Text = new KTextBrowser( this, "TextView" );
    Text->setText( content );
    Text->setNotifyClick( true );
    QHBoxLayout *top = new QHBoxLayout(this);
    top->addWidget(Text);
    setBackgroundColor( Qt::white );
    setFixedSize( 800, 600 );
    top->activate();

    QFont font;
    font.setPixelSize(17);
    font.setFamily("sans serif");
    Text->setFont( font );
    Text->adjustSize();
    Text->updateScrollBars();

    top->activate();

    connect( Text, SIGNAL(urlClick(const QString&)), this, SLOT(urlClick(const QString&)) );
}

void SUSEgreetings::showEvent(QShowEvent* e)
{
    QDialog::showEvent(e);

    adjustSize();
}

void SUSEgreetings::adjustSize()
{
    if (Text->contentsHeight() >= 598) {
        QFont font = Text->font();
        if (font.pixelSize() < 14)
            return;

        font.setPixelSize(font.pixelSize()-1);
        Text->setFont(font);

        QTimer::singleShot(0, this, SLOT(adjustSize()));
    }
}

SUSEgreetings::~SUSEgreetings()
{
}

void SUSEgreetings::urlClick(const QString &s){
    // to get rid of const
    QString url(s);

    if (url.startsWith("/close")) {
        kapp->quit();
	return;
    }

    if (url=="http://software.opensuse.org/")
      url="http://software.opensuse.org/search?baseproject=openSUSE%3A10.3";

    KProcess p;

    if ( url.startsWith( "/you" ) ) {
        p << "kdesu" << "/sbin/yast2" << "online_update";
    } else if ( url.startsWith( "irc://") ) {
        p << "konversationircprotocolhandler" << url;
    } else {
        // fallbacks to english version
        if ( url.contains("/susetour/") )
        {
            QFile file(url);
            if(!file.exists())
            {
                url = DEFAULT_SUSETOUR;
            }
        }
        else if ( url.contains("/RELEASE-NOTES") )
        {
            QFile file(url);
            if(!file.exists())
            {
                url = DEFAULT_RELEASENOTES;
            }
        }

        p << "kfmclient" << "openURL" << url;

    }

    // give the user some feedback
    QApplication::setOverrideCursor(waitCursor);
    QTimer::singleShot(2000, this, SLOT(resetCursor()));

    p.start( KProcess::DontCare );
};

void SUSEgreetings::resetCursor(){
    QApplication::restoreOverrideCursor();
};

int SUSEgreetings::buttons()
{
    return 0 ;
}

QString SUSEgreetings::quickHelp() const
{
    return QString("");
}

#include "greetings.moc"
